package ru.bitel.bgbilling.modules.tv.dyn.tv24h;

import java.net.URL;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.json.JSONArray;
import org.json.JSONObject;

import ru.bitel.bgbilling.kernel.module.common.bean.User;
import ru.bitel.bgbilling.kernel.script.server.dev.GlobalScriptBase;
import ru.bitel.bgbilling.modules.tv.api.common.bean.TvAccount;
import ru.bitel.bgbilling.modules.tv.api.common.bean.TvDevice;
import ru.bitel.bgbilling.modules.tv.api.server.bean.TvAccountDao;
import ru.bitel.bgbilling.modules.tv.api.server.bean.TvDeviceDao;
import ru.bitel.bgbilling.modules.tv.dyn.JsonClient;
import ru.bitel.bgbilling.modules.tv.dyn.JsonClient.Method;
import ru.bitel.bgbilling.server.util.ModuleSetup;
import ru.bitel.bgbilling.server.util.Setup;
import ru.bitel.common.sql.ConnectionSet;
import ru.bitel.oss.systems.inventory.product.common.bean.Product;
import ru.bitel.oss.systems.inventory.product.common.bean.ProductSpec;
import ru.bitel.oss.systems.inventory.product.server.bean.ProductDao;
import ru.bitel.oss.systems.inventory.product.server.bean.ProductSpecDao;

public class Tv24hSync
    extends GlobalScriptBase
{
    @Override
    public void execute( Setup setup, ConnectionSet connectionSet )
        throws Exception
    {
        final int moduleId = 13;
        ModuleSetup moduleSetup = setup.getModuleSetup( moduleId );
        String providerURL = moduleSetup.get( "om.tv24h.providerURL", "https://api.24h.tv/" );
        JsonClient jsonClient = new JsonClient( new URL( providerURL ), null, null );
        
        try( TvDeviceDao tvDeviceDao = new TvDeviceDao( connectionSet.getConnection(), moduleId );
             TvAccountDao tvAccountDao = new TvAccountDao( connectionSet.getConnection(), moduleId );
             ProductDao productDao = new ProductDao( connectionSet.getConnection(), User.USER_SERVER );
             ProductSpecDao productSpecDao = new ProductSpecDao( connectionSet.getConnection() ) )
        {
            TvDevice tvDevice = tvDeviceDao.get( 3 );
            String token = tvDevice.getSecret();
            // System.out.println( "token = " + token );
            
            Map<String, String> requestOptions = new HashMap<>();
            requestOptions.put( "Content-Type", "application/json" );
            requestOptions.put( "Accept", "application/json" );
            
            int offset = 0,  count = 1;
            while ( true )
            {
                String resource = "/v2/users?token=" + token + "&subscriptions=current&limit=100&offset=" + offset;
                JSONArray users = jsonClient.invokeAndGetArray( Method.get, requestOptions, resource, null, null );
                for ( int index = 0, len = users.length(); index < len; index++ )
                {
                    JSONObject user = users.getJSONObject( index );
                    JSONArray subscriptions = user.optJSONArray( "subscriptions" );
                    String userName = user.optString( "username" );
                    TvAccount tvAccount = tvAccountDao.get( userName, new Date() );
                    if ( subscriptions.length() > 0 && tvAccount != null )
                    {
                        List<Product> productList = productDao.list( moduleId, tvAccount.getContractId(), tvAccount.getId(), false, new Date() );
                        for ( int j = 0, jl = subscriptions.length(); j < jl; j++ )
                        {
                            JSONObject subscription = subscriptions.optJSONObject( j );
                            int subscriptionId = subscription.getJSONObject( "packet" ).getInt( "id" );
                            ProductSpec productSpec = productSpecDao.getByIdentifier( String.valueOf( subscriptionId  ) );
                            if ( productSpec == null )
                            {
                                System.out.println( "ERROR!!! productSpec == null [subscription =" + subscription.toString() + "] " );
                                continue;
                            }
                            for ( Product product : productList )
                            {
                                if ( product.getProductSpecId() == productSpec.getId() )
                                {
                                    productList.remove( product );
                                    break;
                                }
                            }
                        }
                        
                        if ( !productList.isEmpty() )
                        {
                            System.out.println( "\n" + count + ") " + user.toString() );                
                            System.out.println( "\n" + "tvAccount = " + ( tvAccount == null ? null : tvAccount.toString() ) );
                            System.out.println( "\nProducts: " );
                            for ( Product product : productList )
                            {
                                System.out.println( "\tProduct -> " + product );                
                            }
                            count++;
                        }
                    }
                }
                //break;
                if ( users.length() == 0 )
                {
                    break;
                }
                offset += 100;
            }
        }
        
        jsonClient.disconnect();
    }
}
//16) {
//    "subscriptions":
//    [
//         {
//             "start_at":"2020-03-30T14:47:35.492829Z",
//             "end_at":"2020-04-13T14:47:35.492840Z",
//             "pauses":[],
//             "is_paused":false,
//             "id":"429335901080912206",
//             "renew":false,
//             "packet":
//             {
//                 "include":null,
//                 "period":"1-0-0",
//                 "agreement":"",
//                 "price":"0.00",
//                 "name":"Промо",
//                 "available":null,
//                 "short":null,
//                 "description":"Промо пакет",
//                 "id":5,
//                 "base":true
//             },
//             
//        }
//    ],
//    "id":440,
//    "date_joined":"2020-03-30T14:47:35.453936Z",
//    "first_name":"",
//    "last_name":"",
//    "email":null,
//    "username":"79622844025"
//    "is_active":true,
//    "is_guest":false,
//    "timezone":null,
//    "device_limit":5,
//    "last_login":"2020-03-30T14:48:09.261426Z",
//    "provider_uid":null,
//    "is_validated":true,
//    "tags":[],
//    "is_provider_free":true,
//    "phone":"79622844025",
//    "provider":
//    {
//        "proxy":null,
//        "landing":
//        {
//            "logo":"http://media.ks.platform24.tv/landing/logo/f0/bc/f0bc698221cd6563b8a90d96dfd53206.png",
//            "login":
//            {
//                "description":"Приложение для просмотра интерактивного ТВ от компании Кабельные системы",
//                "title":"Кабельные системы"
//            },
//            "shortname":"",
//            "support":
//            {
//                "phone":"+7(4162)9999-00",
//                "email":"info@dvsat.ru",
//                "url":"www.dvsat.ru"
//            },
//            "url":""
//        },
//        "auto_auth":"",
//        "name":"Кабельные системы",
//        "id":1
//    },
// }
//
//17) {"subscriptions":[{"end_at":"2021-04-15T10:09:26.042602Z","pauses":[],"is_paused":false,"id":"556899369341682579","renew":false,"packet":{"include":null,"period":"0-1-0","agreement":"","price":"300.00","name":"Стандарт","available":null,"short":null,"description":"Полный пакет цифрового тв от Кабельных систем в который входят все доступные каналы для просмотра ТВ","id":16,"base":true},"start_at":"2021-03-17T15:00:00Z"},{"end_at":"2021-03-17T14:59:59Z","pauses":[],"is_paused":false,"id":"546752581288854518","renew":false,"packet":{"include":null,"period":"0-1-0","agreement":"","price":"300.00","name":"Стандарт","available":null,"short":null,"description":"Полный пакет цифрового тв от Кабельных систем в который входят все доступные каналы для просмотра ТВ","id":16,"base":true},"start_at":"2021-02-17T15:00:00Z"},{"end_at":"2021-02-17T14:59:59Z","pauses":[],"is_paused":false,"id":"535518596323869220","renew":false,"packet":{"include":null,"period":"0-1-0","agreement":"","price":"300.00","name":"Стандарт","available":null,"short":null,"description":"Полный пакет цифрового тв от Кабельных систем в который входят все доступные каналы для просмотра ТВ","id":16,"base":true},"start_at":"2021-01-17T15:00:00Z"},{"end_at":"2021-01-17T14:59:59Z","pauses":[],"is_paused":false,"id":"535372841567125956","renew":false,"packet":{"include":null,"period":"0-1-0","agreement":"","price":"300.00","name":"Стандарт","available":null,"short":null,"description":"Полный пакет цифрового тв от Кабельных систем в который входят все доступные каналы для просмотра ТВ","id":16,"base":true},"start_at":"2021-01-16T15:00:00Z"},{"end_at":"2020-04-14T08:49:45.982891Z","pauses":[],"is_paused":false,"id":"429608239294841687","renew":false,"packet":{"include":null,"period":"1-0-0","agreement":"","price":"0.00","name":"Промо","available":null,"short":null,"description":"Промо пакет","id":5,"base":true},"start_at":"2020-03-31T08:49:45.982882Z"}],"is_active":true,"is_guest":false,"timezone":null,"device_limit":5,"last_login":"2021-07-09T05:14:16.642910Z","provider_uid":"bgb206","last_name":"","is_validated":false,"tags":[],"is_provider_free":true,"phone":"79246820182","provider":{"proxy":null,"landing":{"logo":"http://media.ks.platform24.tv/landing/logo/f0/bc/f0bc698221cd6563b8a90d96dfd53206.png","login":{"description":"Приложение для просмотра интерактивного ТВ от компании Кабельные системы","title":"Кабельные системы"},"shortname":"","support":{"phone":"+7(4162)9999-00","email":"info@dvsat.ru","url":"www.dvsat.ru"},"url":""},"auto_auth":"","name":"Кабельные системы","id":1},"id":771,"date_joined":"2020-03-31T08:49:45.944523Z","first_name":"","email":null,"username":"79246820182"}
